package org.agilefaqs.rft;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

import java.util.ArrayList;
import java.util.List;

import org.agilefaqs.rft.internal.Domain;
import org.agilefaqs.rft.internal.DomainFactory;
import org.agilefaqs.rft.internal.IPAddress;
import org.junit.Test;

public class IPAddressExtractorTest {
    private static final FakeNetwork FAKE_NETWORK = new FakeNetwork();
    private static final Domain GMAIL = DomainFactory.build("gmail.com", FAKE_NETWORK);
    private List<EmailHeader> headers = new ArrayList<EmailHeader>();
    private IPAddress senderIP;
    private int distance;

    @Test
    public void mineSenderFromEdgeServerRecordInMailHeaders() {
        header(
                mail_from("67.205.47.130", "agileindia.org").receivedBy("75.119.213.4", "mails.agileindia.org"),
                mail_from(null, "mail-vw0-f172.google.com").receivedBy("67.205.47.130", "agileindia.org"));
        senderIP = extractSenderIPFrom(headers).forRecipientDomain(GMAIL);
        assertThat(senderIP).is("209.85.212.172");
        assertDistanceOfMatchingHeaderFromTopIs(2);
    }

    @Test
    public void useSenderIPForInvalidSenderEdgeServerDomainName() {
        header(
                mail_from("67.205.47.130", "agileindia.org").receivedBy("75.119.213.4", "mails.agileindia.org"),
                mail_from("209.85.212.172", "cannot-exist.agilefaqs.com").receivedBy("67.205.47.130", "agileindia.org"));
        senderIP = extractSenderIPFrom(headers).forRecipientDomain(GMAIL);
        assertThat(senderIP).is("209.85.212.172");
        assertDistanceOfMatchingHeaderFromTopIs(2);
    }

    @Test
    public void senderNameCanBeIPAddress() {
        header(mail_from(null, "209.85.212.172").receivedBy("67.205.47.130", "agileindia.org"));
        senderIP = extractSenderIPFrom(headers).forRecipientDomain(GMAIL);
        assertThat(senderIP).is("209.85.212.172");
        assertDistanceOfMatchingHeaderFromTopIs(1);
    }

    @Test
    public void matchMXRecordIPWithReciever() {
        header(mail_from("", "mail-vw0-f172.google.com").receivedBy("67.205.47.130", "apache2-echo.robin.dreamhost.com"));
        senderIP = extractSenderIPFrom(headers).forRecipientDomain(GMAIL);
        assertThat(senderIP).is("209.85.212.172");
        assertDistanceOfMatchingHeaderFromTopIs(1);
    }

    @Test
    public void skipHeaderRecordsThatDontCrossEdgeServers() {
        header(
                mail_from("192.168.1.47", "smtp.gmail.com").receivedBy("75.119.213.4", "mail.gmail.com"),
                mail_from("192.168.1.3", "192.168.6.242").receivedBy("192.168.1.47", "smtp.gmail.com"));
        senderIP = extractSenderIPFrom(headers).forRecipientDomain(GMAIL);
        assertFalse(senderIP.isValid());
        assertDistanceOfMatchingHeaderFromTopIs(0);
    }

    private IPAddressExtractorTest extractSenderIPFrom(final List<EmailHeader> headers) {
        this.headers = headers;
        return this;
    }

    private void is(final String expectedIp) {
        assertEquals(expectedIp, senderIP.toString());
    }

    private IPAddressExtractorTest assertThat(final IPAddress senderIP) {
        this.senderIP = senderIP;
        return this;
    }

    private IPAddress forRecipientDomain(final Domain domain) {
        IPAddressExtractor addressExtractor = new IPAddressExtractor(headers, domain);
        distance = addressExtractor.getDistance();
        return addressExtractor.getSenderIP();
    }

    private EmailHeader receivedBy(final String ip, final String domainName) {
        return new EmailHeader(fromIp, fromDomain, ip, domainName, FAKE_NETWORK);
    }

    private String fromIp;
    private String fromDomain;

    private IPAddressExtractorTest mail_from(final String ip, final String domainName) {
        fromIp = ip;
        fromDomain = domainName;
        return this;
    }

    private void header(final EmailHeader... headers) {
        for (EmailHeader header : headers) {
            this.headers.add(header);
        }
    }

    private void assertDistanceOfMatchingHeaderFromTopIs(final int expected) {
        assertEquals(expected, distance);
    }
}
